/*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// (C) PicoPeta Simputers Pvt. Ltd. (2003)
// Author: Santhosh N

#include "label.h"
#include "font.h"
#include "image.h"
#include "widget_factory.h"
#include "display.h"

#include <X11/extensions/shape.h>
#include <stdlib.h>
#include <string.h>

px_widget* label_create(px_widget* parent, int x, int y, int width, int height, const char* tag)
{
	px_widget* widget = (px_widget*)malloc(sizeof(px_widget));
	Display* display = display_get_Xdisplay();
#ifdef __debug__
	printf("%s() Begin : %p\n", __func__, widget);
#endif

	// Initialize common widget variables

	widget->win = XCreateWindow(display, parent->win, x, y, width, height, 0, CopyFromParent,
			CopyFromParent, CopyFromParent, 0, NULL);


	widget->x = x;
	widget->y = y;
	widget->height = height;
	widget->width = width;

	if (tag != NULL)
	{
		widget->tag = (char*)malloc(sizeof(char) * (strlen(tag) + 1));
		strcpy(widget->tag, tag);
	}
	else
		widget->tag = NULL;

	widget->wtype = LABEL;

	widget->use_pixmap = False;
	widget->background.background_color = (px_color*)malloc(sizeof(px_color));
	widget->background.background_color->type = TRANSPARENT;

	// Initializing backing store
	image_initialize_pixmap(&(widget->backing_store));
	image_create_pixmap(widget->win, &(widget->backing_store), width, height);
#ifdef __debug__
	XSync(display, False);
	printf("%s(): %p - Created Window with parent window = %lu, x = %d, y = %d, width = %d, height = %d, new window = %lu\n\n", 
			__func__, widget, parent->win, x, y, width, height, widget->win);
#endif
	return widget;
}

void label_set_parameters(px_widget* widget)
{
	px_label* label = &(widget->specific.label);

	widget->default_callbacks[BUTTON_PRESSED] = label_default_button_press;
	widget->default_callbacks[BUTTON_RELEASED] = label_default_button_released;
	widget->default_callbacks[MOUSE_ENTER] = label_default_mouse_enter;
	widget->default_callbacks[MOUSE_LEAVE] = label_default_mouse_leave;

	widget->set_size = label_set_size;
	widget->redraw = label_redraw;
	widget->destroy = label_destroy;
	
	
	label->fontname = NULL;
	
	label->normal = NULL;
	label->active = NULL;
	label->disabled= NULL;

	label->layout.just = JUSTIFY_CENTER;
	label->layout.wrap = WRAP_CHAR;
	label->layout.text = NULL;
	label->layout.x = 0;
	label->layout.y = 0;
	label->layout.width = widget->width;
	label->layout.height = widget->height;
	label->layout.font = NULL;
	label->layout.font_color = NULL;
	label->layout.chunks = NULL;
	label->layout.no_of_chunks = 0;
	
	label->change_size = False;

	label->set_text = label_set_text;
	label->set_justification = label_set_justification;
	label->set_font = label_set_font;

	label->get_justification = label_get_justification;
	label->get_text = label_get_text;

	label->set_normal_color = label_set_normal_color;
	label->set_active_color = label_set_active_color;
	label->set_disabled_color = label_set_disabled_color;

	label->set_wrap_type = widget_set_wrap_type;
	label->size_depends_on_contents = widget_size_depends_on_contents;
}

void label_set_size(px_widget* widget, unsigned long width, unsigned long height)
{
	if (!widget->specific.label.change_size)
	{
		widget_set_size(widget, width, height);
		widget->specific.label.layout.width = widget->width;
		widget->specific.label.layout.height = widget->height;

		widget_compute_textlayout(&(widget->specific.label.layout), widget);
	}
}

Bool label_default_button_press(px_widget* widget, px_event* event)
{
	widget->state =  ACTIVE;
	(*widget->redraw)(widget);

	return True;
}

Bool label_default_button_released(px_widget* widget, px_event* event)
{
	widget->state =  NORMAL;
	(*widget->redraw)(widget);
	return True;
}

Bool label_default_mouse_enter(px_widget* widget, px_event* event)
{
	widget->state =  ACTIVE;
	(*widget->redraw)(widget);

	return True;
}

Bool label_default_mouse_leave(px_widget* widget, px_event* event)
{
	widget->state = NORMAL;
	(*widget->redraw)(widget);

	return True;
}

void label_redraw(px_widget* widget)
{
	widget_states state;
	Display* display = display_get_Xdisplay();
	px_label* label = &(widget->specific.label);
#ifdef __debug__
	printf("%s() : Begin %p\n", __func__, widget);
#endif
	
	state = (*widget->get_state)(widget);
	
	if (widget->use_pixmap)
	{
		image_copy_pixmap(widget->win, &(widget->backing_store), widget->background.background_pixmap);
	} 
	else
	{
		widget_draw_px_color(&(widget->backing_store), widget->background.background_color);
	}

	if (state == DISABLED && widget->specific.label.disabled != NULL)
	{
		label->layout.font_color = widget->specific.label.disabled;
	}
	else if (state == ACTIVE && widget->specific.label.active != NULL)
	{
		label->layout.font_color = widget->specific.label.active;
	}
	else if (widget->specific.label.normal != NULL)
	{
		label->layout.font_color = widget->specific.label.normal;
	}
	else
	{
		label->layout.font_color = NULL;
	}


	widget_draw_text(&(label->layout), &(widget->backing_store));
	widget_draw_border(widget);
	XClearWindow(display, widget->win);
	widget_set_event_mask(widget, widget->emask);

#ifdef __debug__
	XSync(display, False);
	printf("%s() End : %p\n", __func__, widget);
#endif
}

void label_destroy(px_widget* widget)
{
	px_label* label = &(widget->specific.label);
	px_textchunk* chunk, *next;

	if (widget->specific.label.layout.text != NULL)
		free(widget->specific.label.layout.text);

	if (widget->specific.label.normal != NULL)
	{
		free(widget->specific.label.normal);
	}
	
	if (widget->specific.label.active != NULL)
	{
		
		free(widget->specific.label.active);
	}

	if (widget->specific.label.disabled != NULL)
	{
		free(widget->specific.label.disabled);
	}

	if (widget->specific.label.fontname != NULL)
	{
		font_free_font(widget->specific.label.fontname);
		free(widget->specific.label.fontname);
	}

	chunk = label->layout.chunks;
	while (chunk != NULL)
	{
		next = chunk->next;
		free(chunk);
		chunk = next;
	}

	widget_destroy(widget);
}

void label_set_normal_color(px_widget* widget, const char* colour)
{
	if (widget->specific.label.normal == NULL)
	{
		widget->specific.label.normal = (XColor*)malloc(sizeof(XColor));
	}
	widget_xcolor_from_str(widget->specific.label.normal, colour);
}


void label_set_active_color(px_widget* widget, const char* colour)
{
	if (widget->specific.label.active == NULL)
	{
		widget->specific.label.active = (XColor*)malloc(sizeof(XColor));
	}
	widget_xcolor_from_str(widget->specific.label.active, colour);
}

void label_set_disabled_color(px_widget* widget, const char* colour)
{
	if (widget->specific.label.disabled == NULL)
	{
		widget->specific.label.disabled = (XColor*)malloc(sizeof(XColor));
	}
	widget_xcolor_from_str(widget->specific.label.disabled, colour);
}

void label_set_text(px_widget* widget, const char* t)
{
	if (widget->specific.label.layout.text != NULL)
	{
		free(widget->specific.label.layout.text);
		widget->specific.label.layout.text = NULL;
	};

	if (t != NULL)
	{
		widget->specific.label.layout.text = strdup(t);
		widget_compute_textlayout(&(widget->specific.label.layout), widget);
		if (widget->specific.label.change_size)
		{
			widget_set_size(widget, widget->specific.label.layout.width < 1 ? 1 : widget->specific.label.layout.width,
					widget->specific.label.layout.height < 1 ? 1 : widget->specific.label.layout.height);
		}
	}
}

void label_set_font(px_widget* widget, const char* fontname)
{
	if (widget->specific.label.fontname != NULL)
	{
		font_free_font(widget->specific.label.fontname);
		free(widget->specific.label.fontname);
		widget->specific.label.fontname = NULL;
	}

	widget->specific.label.fontname = strdup(fontname);
	widget->specific.label.layout.font = font_load_font(widget->specific.label.fontname);
	widget_compute_textlayout(&(widget->specific.label.layout), widget);
	if (widget->specific.label.change_size)
	{
		widget_set_size(widget, widget->specific.label.layout.width < 1 ? 1 : widget->specific.label.layout.width,
				widget->specific.label.layout.height < 1 ? 1 : widget->specific.label.layout.height);
	}

}

void label_set_justification(px_widget* widget, justification j)
{
	widget->specific.label.layout.just = j;
}

justification label_get_justification(const px_widget* widget)
{
	return widget->specific.label.layout.just;
}

const char* label_get_text(const px_widget* widget)
{
	return widget->specific.label.layout.text;
}

